package com.ibm.tools.attach.javaSE;
/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2009, 2010  All Rights Reserved
 */

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

class Advertisement {
	private static final String KEY_ATTACH_NOTIFICATION_SYNC = "attachNotificationSync"; //$NON-NLS-1$
	private static final String KEY_REPLY_FILE = "replyFile"; //$NON-NLS-1$
	private static final String KEY_NOTIFIER = "notifier"; //$NON-NLS-1$
	private static final String KEY_DISPLAY_NAME = "displayName"; //$NON-NLS-1$
	private static final String KEY_VM_ID = "vmId"; //$NON-NLS-1$
	private static final String KEY_USER_UID = "userUid"; //$NON-NLS-1$
	private static final String KEY_USER_ID = "userId"; //$NON-NLS-1$
	private static final String KEY_VERSION = "version"; //$NON-NLS-1$
	private static final String KEY_PROCESS_ID = "processId"; //$NON-NLS-1$
	private static final String ADVERT_FILENAME = "attachInfo"; //$NON-NLS-1$
	private Properties props;
	private final long pid, uid;

	/**
	 * Create these objects via readAdvertisementFile().
	 * @param advertStream
	 * @throws IOException
	 */
	private Advertisement(InputStream advertStream) throws IOException {
		props = new Properties();
		try {
			props.load(advertStream);
		} catch (IllegalArgumentException e) {
			// K0556 = Error parsing virtual machine advertisement file
			throw (new IOException(com.ibm.oti.util.Msg.getString("K0556"), e));
		}
		long id;
		try {
			id = Long.parseLong(props.getProperty(KEY_PROCESS_ID));
		} catch (NumberFormatException e) {
			id = 0;
		}
		pid = id;
		try {
			id = Long.parseLong(props.getProperty(KEY_USER_UID));
		} catch (NumberFormatException e) {
			id = 0;
		}
		if ((null == props.getProperty(KEY_VM_ID)) || (null == props.getProperty(KEY_DISPLAY_NAME))) {
			/* CMVC 160738 : handle malformed advertisement files */
			// K0554 = Virtual machine ID or display name is null
			throw new IOException(com.ibm.oti.util.Msg.getString("K0554")); //$NON-NLS-1$
		}
		uid = id;
	}

	/**
	 * Factory method to read the data from an advertisement file
	 * @param advertStream InputStream to an open file
	 * @return Advertisement object containing the data from the file.
	 * @throws IOException
	 */
	static Advertisement readAdvertisementFile(InputStream advertStream) throws IOException {
		Advertisement advert = null;
		try {
			advert = new Advertisement(advertStream);
		} catch (IOException e) {
			IPC.logMessage("IOException opening advertisement ", e.getMessage());  //$NON-NLS-1$
			throw e;
		}
		return advert;
	}
	/**
	 * @param vmId machine-friendly name of the VM
	 * @param displayName human-friendly name of the VM. If null or zero-length, vmId is used
	 * @return Create the information to discover and notify this VM
	 */
	@SuppressWarnings("nls")
	private static StringBuilder createAdvertContent(String vmId, String displayName) {
		StringBuilder contentBuffer = new StringBuilder(512);
		addKeyAsciiValue(contentBuffer, KEY_VERSION, "0.1");
		addKeyValue(contentBuffer, KEY_USER_ID ,System.getProperty("user.name"));
		/* CMVC 161414 - PIDs and UIDs are long */
		addKeyAsciiValue(contentBuffer, KEY_USER_UID ,Long.toString(IPC.getUid()));
		addKeyAsciiValue(contentBuffer, KEY_PROCESS_ID, Long.toString(IPC.getProcessId()));
		addKeyValue(contentBuffer, KEY_VM_ID, vmId);
		addKeyValue(contentBuffer, KEY_DISPLAY_NAME, (((null == displayName) || (displayName.length() == 0))? vmId: displayName));
		addKeyValue(contentBuffer, KEY_NOTIFIER, CommonDirectory.getSemaphoreName());
		addKeyValue(contentBuffer, KEY_REPLY_FILE, Reply.getReplyPath(vmId));
		addKeyValue(contentBuffer, KEY_ATTACH_NOTIFICATION_SYNC, TargetDirectory.getNotificationSyncFile(vmId).getAbsolutePath());
		return contentBuffer;
	}

	/**
	 * Add a key-value pair to a string buffer in Properties format.
	 * Cannot use Properties because it causes several additional classes to be loaded (defect 158165)
	 * @param contentBuffer  where to put the string
	 * @param newKey key to identify the data
	 * @param newValue value of the data
	 */
	private static void addKeyValue(StringBuilder contentBuffer, String newKey, String newValue) {
		contentBuffer.append(newKey);
		contentBuffer.append('=');
		encodeString(contentBuffer, newValue);
		contentBuffer.append('\n');
	}

	/**
	 * Add a key-value pair to a string buffer in Properties format.
	 * Use when newValue does not contain control characters
	 * @param contentBuffer  where to put the string
	 * @param newKey key to identify the data
	 * @param newValue value of the data
	 */
	private static void addKeyAsciiValue(StringBuilder contentBuffer, String newKey, String newValue) {
		contentBuffer.append(newKey);
		contentBuffer.append('=');
		contentBuffer.append(newValue);
		contentBuffer.append('\n');
	}
	/**
	 * Write out file with control characters escaped
	 * @param buffer output buffer with escaped characters
	 * @param originalString original data
	 */
	private static void encodeString(StringBuilder buffer, String originalString) {
		for (int i=0; i < originalString.length(); i++) {
			char ch = originalString.charAt(i);
			switch (ch) {
			case '\t':
				buffer.append("\\t"); //$NON-NLS-1$
				break;
			case '\n':
				buffer.append("\\n"); //$NON-NLS-1$
				break;
			case '\f':
				buffer.append("\\f"); //$NON-NLS-1$
				break;
			case '\r':
				buffer.append("\\r"); //$NON-NLS-1$
				break;
			default:
				if ("\\#!=:".indexOf(ch) >= 0 || (ch == ' ')) //$NON-NLS-1$
					buffer.append('\\');
				if (ch >= ' ' && ch <= '~') {
					buffer.append(ch);
				} else {
					String hex = Integer.toHexString(ch);
					buffer.append("\\u"); //$NON-NLS-1$
					for (int j=0; j < 4 - hex.length(); j++) buffer.append("0"); //$NON-NLS-1$
					buffer.append(hex);
				}
			}
		}
	}

	/**
	 * Write an advertisement file. Adds escapes where necessary.
	 * @param vmId ID of this VM
	 * @param displayName display name of this VM
	 * @throws IOException if cannot open the advertisement file
	 */
	static void createAdvertisementFile(String vmId, String displayName) throws IOException {

		String advertFilePath=(new File(TargetDirectory.getTargetDirectoryPath(vmId),Advertisement.ADVERT_FILENAME)).getPath();
		File advertFile = new java.io.File(advertFilePath);
		if (AttachHandler.isAttachApiTerminated()) {
			/* don't create the file if we don't need it */
			return;
		}
		IPC.createFileWithPermissions(advertFile.getAbsolutePath(), TargetDirectory.ADVERTISEMENT_FILE_PERMISSIONS);
		/* AttachHandler.terminate() will delete this file on shutdown */
		FileOutputStream advertOutputStream = new FileOutputStream(advertFile);
		try {
			StringBuilder advertContent = createAdvertContent(vmId, displayName);
			advertOutputStream.write(advertContent.toString().getBytes("ISO8859_1")); //$NON-NLS-1$
			if (IPC.loggingEnabled ) {
				IPC.logMessage("createAdvertisementFile ", advertFile.getAbsolutePath()); //$NON-NLS-1$
			}
		} finally {
			advertOutputStream.close(); /* cleanup from findbugs */
		}
	}

	/**
	 * Remove the advertisement files and the directory containing it.
	 * @param myVmId ID of this VM
	 */
	static void deleteAdvertisementDir(String myVmId) {
		if ((null != myVmId) && (0 != myVmId.length())) { /* skip if the vmid was never set - we didn't create the directory */
			File notDir = new File(TargetDirectory.getTargetDirectoryPath(myVmId));
			File[] vmFiles = notDir.listFiles();
			if (IPC.loggingEnabled ) {
				IPC.logMessage("deleting directory ", notDir.getAbsolutePath()); //$NON-NLS-1$
			}
			if (null != vmFiles) {
				for (File f: vmFiles) {
					if (!f.delete()) {
						IPC.logMessage("error deleting directory ", f.getAbsolutePath()); //$NON-NLS-1$
					} else if (IPC.loggingEnabled ) {
						IPC.logMessage("deleted file ", f.getAbsolutePath()); //$NON-NLS-1$
					}
				}
			}
			if (!notDir.delete()) {
				IPC.logMessage("error deleting directory ", notDir.getAbsolutePath()); //$NON-NLS-1$
			} else if (IPC.loggingEnabled ) {
				IPC.logMessage("deleted directory ", notDir.getAbsolutePath()); //$NON-NLS-1$
			}
		}
	}

	/**
	 * @return name of advertisement file, not including directory path
	 */
	static String getFilename() {
		return Advertisement.ADVERT_FILENAME;
	}

	/**
	 * @return Target virtual machine's display name
	 */
	public String getDisplayName() {
		return props.getProperty(KEY_DISPLAY_NAME);
	}

	/**
	 *
	 * @return Operating system process ID of the target VM
	 */
	public long getProcessId() {

		return pid;
	}

	/**
	 *
	 * @return machine-friendly identifier of the target VM
	 */
	public String getVmId() {
		return props.getProperty(KEY_VM_ID);
	}

	/**
	 *
	 * @return identifier of the semaphore used to notifiy the target
	 */
	public String getNotifier() {
		return props.getProperty(KEY_NOTIFIER);
	}

	/**
	 * attacher creates a file using this path when it is attaching to the target.
	 * @return file path
	 */
	public String getReplyFile() {
		return props.getProperty(KEY_REPLY_FILE);
	}

	/**
	 * attach locks the sync file to prevent a target from consuming >1 semaphore increments
	 * @return file path
	 */
	public String getNotificationSync() {
		return props.getProperty(KEY_ATTACH_NOTIFICATION_SYNC);
	}

	/**
	 *
	 * @return user identifier (numeric)
	 */
	public long getUid() {
		return uid;
	}

}
